@@ -6,7 +6,7 @@
 # If the MCMC simulation has been run for only one chain report the Geweke diagnostic (Z-score)
 #
 
-Id <- "$Id: c212.interim.BB.hier3.lev1.convergence.R,v 1.8 2016/12/19 11:36:43 clb13102 Exp clb13102 $"
+Id <- "$Id: c212.interim.BB.hier3.lev1.convergence.R,v 1.9 2019/05/05 13:18:12 clb13102 Exp clb13102 $"
 
 c212.interim.BB.dep.lev1.convergence.diag <- function(raw, debug_diagnostic = FALSE)
 {
@@ -614,12 +614,3 @@ c212.interim.BB.dep.lev1.print.convergence.summary <- function(conv) {
 			}
 	}
 }
-
-chk_val <- function(val, q = 0.975) {
-	if (abs(val) > qnorm(q)) {
-		return("*")
-	}
-	else {
-		return("-")
-	}
-}
